import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.io.*;

/**
 * Controls is the class which contains all the buttons as well as their
 * action events.
 *
 * @author Hussein Gibbins
 */
public class Controls {

	private JFrame parentFrame;
	private MapPanel map;
	private TestbedManager tbm;
	private JPanel cardPanel;

	private JButton addButton;
	private JButton editButton;
	private JButton viewButton;
	private JButton optionsButton;
	private JButton quitButton;
	private JButton saveButton;

	/**
	 * Creates an instance of the Control object which can be used to produce a menu bar.
	 */
	public Controls(JFrame parentFrame, MapPanel map, TestbedManager tbm, JPanel cardPanel) {
		this.parentFrame = parentFrame;
		this.map = map;
		this.tbm = tbm;
		this.cardPanel = cardPanel;
	}

	/**
	 * Creates an instance of the menu bar containing all menus and menu items and their actions.
	 *
	 * @return the menu bar.
	 */
	public JMenuBar createMenuBar() {

		JMenuBar menuBar;
		JMenu menu;
		JMenuItem menuItem;
		JRadioButtonMenuItem rbMenuItem;
		JCheckBoxMenuItem cbMenuItem;

		//Create the menu bar.
		menuBar = new JMenuBar();

		//Build the first menu.
		menu = new JMenu("File");
		menu.setMnemonic(KeyEvent.VK_F);
		menuBar.add(menu);

		//a group of JMenuItems
		menuItem = new JMenuItem("Open");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {

				JFileChooser openfc = new JFileChooser();
				openfc.setFileSelectionMode(JFileChooser.DIRECTORIES_ONLY);

				//SHOW
				int returnVal = openfc.showDialog(new JFrame(), "Open");

				//Process the results.
				if (returnVal == JFileChooser.APPROVE_OPTION) {

					File file = openfc.getSelectedFile();
					Properties props = checkPath(file.getPath()); //check if valid web-app. //open props file for writing.
		
					if (props != null) {


						Properties thisProps = new Properties();

						try {
							thisProps.load(new FileInputStream("./testbed.properties"));
						} catch (FileNotFoundException fnfe) {
							JOptionPane.showMessageDialog(null,
								"Failed to Open application properties file for writing");
							return;
						} catch (IOException ioe) {
							JOptionPane.showMessageDialog(null,
							"Error with properties file located in application directory");
							return;
						}

						//update tbm.
						thisProps.setProperty("webapp_path", file.getPath() + "/");

						try {
							thisProps.store(new FileOutputStream("./testbed.properties"), "props");
						} catch (Exception e) {
							JOptionPane.showMessageDialog(new JFrame(), "error writing properties");
							return;
						}

						// update location list.
						tbm.updateLocationList();
					}

					
					
				} else {
					//System.out.println("Attachment cancelled by user.");
				}
			}
		});
		menu.add(menuItem);

		menuItem = new JMenuItem("Save");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				tbm.saveLocations();
				//System.out.println("saved...");
			}
		});
		menu.add(menuItem);

		menu.addSeparator();

		menuItem = new JMenuItem("Exit");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				System.exit(0);
			}
		});
		menu.add(menuItem);


		//Build the second menu.
		menu = new JMenu("Mode");
		menu.setMnemonic(KeyEvent.VK_M);
		menuBar.add(menu);

		//a group of radio button menu items
		ButtonGroup group = new ButtonGroup();

		rbMenuItem = new JRadioButtonMenuItem("View");
		rbMenuItem.setSelected(true);
		group.add(rbMenuItem);
		rbMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				map.setStatus(MapPanel.VIEWING);

				CardLayout cl = (CardLayout)cardPanel.getLayout();
				cl.show(cardPanel, Constants.VIEW_PANEL);
				tbm.queryTop();
			}
		});
		menu.add(rbMenuItem);

		rbMenuItem = new JRadioButtonMenuItem("Edit");
		group.add(rbMenuItem);
		rbMenuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {

					map.setStatus(MapPanel.EDITING);

					CardLayout cl = (CardLayout)cardPanel.getLayout();
					cl.show(cardPanel, Constants.EDIT_PANEL);
			}
		});
		menu.add(rbMenuItem);


		//Build the third menu.
		menu = new JMenu("Options");
		menu.setMnemonic(KeyEvent.VK_O);
		menuBar.add(menu);

		//a group of JMenuItems
		menuItem = new JMenuItem("Testbed Name");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				JDialog name_dialog = new NameDialog(parentFrame, true, tbm);
				name_dialog.setLocationRelativeTo(parentFrame);
				name_dialog.show();
			}
            });
		menu.add(menuItem);

		menuItem = new JMenuItem("Testbed Logo");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				JDialog logo_dialog = new LogoDialog(parentFrame, true, tbm);
				logo_dialog.setLocationRelativeTo(parentFrame);
				logo_dialog.show();
			}
            });
		menu.add(menuItem);

		menuItem = new JMenuItem("Testbed Map");
		menuItem.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				MapDialog map_dialog = new MapDialog(parentFrame, true, tbm);
				map_dialog.setLocationRelativeTo(parentFrame);
				map_dialog.show();
				if (map_dialog.isAccepted()) {
					map.setBackgroundMap(map_dialog.getMapPath());
				}
			}
            });
		menu.add(menuItem);


		return menuBar;
	}

	private Properties checkPath(String path) {

		Properties props = new Properties();

		try {
			props.load(new FileInputStream(path +
				"/WEB-INF/classes/gridslab/testbed.properties"));
		} catch (FileNotFoundException fnfe) {
			JOptionPane.showMessageDialog(null,
				"Failed to open location ("
				+ path
				+ "/testbed.properties) for writing. Please specify the location of a valid web app.");
			return null;
		} catch (IOException ioe) {
			JOptionPane.showMessageDialog(null,
				"Error with properties file located in web app directory ("
				+ path + ")");
			return null;
		}

		return props;
	}
}

