import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;
import javax.naming.*;
import javax.naming.directory.*;

/**
 * InfoFrame defines the text boxes where details of a grid can be added,
 * such as name, address of host, port and any other details. It implements
 * observer, and is observer to the LocationList class.
 *
 * @author Hussein Gibbins.
 */
public class InfoFrame extends JPanel implements Observer {
	
	JButton update;
	JButton delete;
	JTextField name_l = new JTextField();
	JTextField adr = new JTextField();
	JTextField prt = new JTextField();
	JTextArea det = new JTextArea(3, 0);
	JComboBox cntry;
	JComboBox baseDN = new JComboBox();

	private Location loc = null;
	private LocationList ll;

	/**
	 * Creates and initialises an InfoFrame.
	 */
	public InfoFrame() {
		init();
	}

	private void init() {
		setLayout(new BorderLayout());

		/* North Panel */
		JPanel n = new JPanel();
		cntry = new JComboBox(new CountryComboBoxModel());
		baseDN.addItem(Location.DEFAULT_BASEDN);

		add(n, BorderLayout.NORTH);

		n.setLayout(new GridLayout(5,2));

		n.add(new JLabel("Enter name of host"));
		n.add(name_l);
		n.add(new JLabel("Enter Address of host"));
		n.add(adr);
		n.add(new JLabel("Enter port number"));
		n.add(prt);
		n.add(new JLabel("Select Country"));
		n.add(cntry);
		n.add(new JLabel("Select Base DN"));
		JButton baseDNButton = new JButton("Query");
		JPanel baseDNPanel = new JPanel();
		baseDNPanel.setLayout(new BoxLayout(baseDNPanel, BoxLayout.X_AXIS));
		baseDNPanel.add(baseDN);
		baseDNPanel.add(baseDNButton);
                baseDNButton.setActionCommand("queryBaseDN");
                baseDNButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent event) {
                                windowAction(event);
                        }
                });

		n.add(baseDNPanel);

		JPanel c = new JPanel();
		add(c, BorderLayout.CENTER);
		c.setLayout(new BoxLayout(c, BoxLayout.Y_AXIS));
		JLabel gridDetLabel = new JLabel("Enter grid details");
		gridDetLabel.setAlignmentX(Component.CENTER_ALIGNMENT);
		c.add(gridDetLabel);
		det.setLineWrap(true);
		c.add(new JScrollPane(det));

		JPanel s = new JPanel();
		add(s, BorderLayout.SOUTH);
		s.setLayout(new GridLayout(1,2));

		update = new JButton("Update");
		s.add(update);
		update.setActionCommand("update");
		update.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				windowAction(event);
			}
		});
		update.setEnabled(false);
			
		delete = new JButton("Delete Record");
		s.add(delete);
		delete.setActionCommand("delete");
		delete.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				windowAction(event);
			}
		});
		delete.setEnabled(false);

		setBorder(BorderFactory.createEmptyBorder(15,15,15,15));
	}

	private void updateBaseDNComboBox() {

                GridbusDirectory gbd = new GridbusDirectory();
                DirContext ctx;
                NamingEnumeration nameEnum = null;
                SearchResult sr;
                Attributes attrRes;
                Attribute attr;
		boolean online = false;
		String dn;

		baseDN.removeAllItems();

                try {
                        online = gbd.bind("ldap://" + adr.getText() + ":" + prt.getText());
                        ctx = gbd.getDirContext();

			if (online) {
                        	SearchControls searchControls = new SearchControls();
                        	searchControls.setReturningAttributes(new String[] {"namingcontexts"});
                        	searchControls.setSearchScope(SearchControls.OBJECT_SCOPE);
                        	nameEnum = ctx.search("", "objectclass=*", searchControls);
			}

                        if (nameEnum != null) {
                                while (nameEnum.hasMore()) {
                                        sr = (SearchResult)nameEnum.next();
                                        attrRes = sr.getAttributes();
                                        attr = attrRes.get("namingcontexts");
                                        NamingEnumeration ne = attr.getAll();
                                        while(ne.hasMore()) {
						dn = (String) ne.nextElement();
                                                baseDN.addItem(dn);
                                        }
                                }
                        }

		} catch (Exception e) {
			e.printStackTrace();
		}
	}


	private void windowAction(Object actionCommand) {
		String cmd = null;
		
		if (actionCommand != null) {
			if (actionCommand instanceof ActionEvent) {
				cmd = ((ActionEvent)actionCommand).getActionCommand();
			} else {
				cmd = actionCommand.toString();
			}
		}

		if (cmd == null)	{
			/* do nothing */
		} else if (cmd.equals("queryBaseDN")) {
			updateBaseDNComboBox();
		} else if (cmd.equals("delete")){
			ll.remove(ll.getNewLocation());
		} else if (cmd.equals("update"))	{
			Location currentLoc = ll.getNewLocation();

			String name = name_l.getText();	
			String address = adr.getText();
			String cntryData = "" + cntry.getSelectedItem();
			String selectedBaseDN = "" + baseDN.getSelectedItem();
			String details = det.getText();
			String p = prt.getText(); 
			int port = 0;
			try {
				port = Integer.parseInt(p);
			} catch (NumberFormatException nef){
			}

			currentLoc.setName(name);
			currentLoc.setAddress(address);
			currentLoc.setCountryData(cntryData);
			currentLoc.setDetails(details);
			currentLoc.setPort(port);
			currentLoc.setBaseDN(selectedBaseDN);
		}
	}


	/**
	 * Updates the content of this component to reflect the details of the location
	 * currently being examined.
	 *
	 * @see Observer
	 */
	public void update(Observable o, Object args) {
		
		if (ll == null) {
			update.setEnabled(true);
			delete.setEnabled(true);
		}

		ll = ((LocationList)o);
		Location currentLoc = ll.getNewLocation();
		name_l.setText(currentLoc.getName());
		cntry.setSelectedItem(currentLoc.getCountryCode() + ":" + currentLoc.getCountryName());
		cntry.repaint();
		adr.setText(currentLoc.getAddress()); 
		det.setText(currentLoc.getDetails()); 
		prt.setText("" + currentLoc.getPort());
		baseDN.removeAllItems();
		baseDN.addItem(currentLoc.getBaseDN());
	}
}
