import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;
import java.io.*;
import javax.imageio.*;
import java.awt.image.*;

/**
 * Dialog to allow user to modify the Testbed logo.
 *
 * @author      Hussein Gibbins
 */
public class LogoDialog extends JDialog {
	
	JButton save;
	JButton cancel;
	String logo = TestbedManager.getWebappPath()+"images/testbed_logo.jpg";
	JFileChooser logofc;
	Properties thisProps = new Properties();
	TestbedManager tbm;

	ImageIcon logoImage = null;
	JLabel logoPreview = new JLabel(); 
	JButton logoButton;

        /**
         * Creates a LogoDialog instance with the specified modality.  Will access
         * the testbed specified by the provided testbed manager.
         */
	public LogoDialog(Frame owner, boolean modal, TestbedManager tbm) {
		super(owner, modal);
		this.tbm = tbm;
		init();
	}

	private void init() {

		// load thisprops first and then use getProperty for defaults.

		getContentPane().setLayout(new BorderLayout());
		JPanel options = new JPanel();
		getContentPane().add(options, BorderLayout.NORTH);
		options.setLayout(new GridLayout(2,1));

		// make new panel for logo
		JPanel logoPanel = new JPanel();
		getContentPane().add(logoPanel, BorderLayout.CENTER);
		logoPanel.setLayout(new GridLayout(1, 2));

            logoPanel.add(new JLabel("Logo"));

		updateLogo(TestbedManager.getWebappPath() + "images/testbed_logo.jpg");
                logoButton = new JButton("Browse");
		logoButton.setIcon(logoImage);
		logoButton.setVerticalTextPosition(AbstractButton.BOTTOM);
		logoButton.setHorizontalTextPosition(AbstractButton.CENTER);
                logoPanel.add(logoButton);
                logoButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent event) {

                                //Set up the file chooser.
                                if (logofc == null) {
                                        logofc = new JFileChooser();

                                        //Add a custom file filter and disable the default
                                        //(Accept All) file filter.
                                        logofc.addChoosableFileFilter(new ImageFilter());
                                        logofc.setAcceptAllFileFilterUsed(false);

                                        //Add the preview pane.
                                        logofc.setAccessory(new ImagePreview(logofc));
                                }

                                //Show it.
                                int returnVal = logofc.showDialog(new JFrame(), "Set");

                                //Process the results.
                                if (returnVal == JFileChooser.APPROVE_OPTION) {
                                        File file = logofc.getSelectedFile();
					logo = file.getPath();
                                        updateLogo(file.getPath());
					logoButton.setIcon(logoImage);
                                } else {
                                        //System.out.println("Attachment cancelled by user.");
                                }

                                //Reset the file chooser for the next time it's shown.
                                logofc.setSelectedFile(null);
                        }
                });


		//do buttons
		JPanel buttons = new JPanel();
		getContentPane().add(buttons, BorderLayout.SOUTH);
		buttons.setLayout(new BoxLayout(buttons, BoxLayout.X_AXIS));

                JButton save = new JButton("Save");
                buttons.add(save);
                save.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent event) {
					save();
                        }
                });

                JButton cancel = new JButton("Cancel");
                buttons.add(cancel);
                cancel.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent event) {
                                dispose();
                        }
                });

		this.pack();
	}

	private void updateLogo(String logoPath) {

                ImageIcon tmpIcon = new ImageIcon(logoPath);
                ImageIcon tmpIconx = new ImageIcon(logoPath);
                ImageIcon tmpIconxy = new ImageIcon(logoPath);

                if (tmpIcon != null) {
                        if (tmpIcon.getIconWidth() > 90) {
                                tmpIconx = new ImageIcon(tmpIcon.getImage().
                                                getScaledInstance(90, -1,
                                                Image.SCALE_DEFAULT));
				logoImage = tmpIconx;
            		} if (tmpIconx.getIconHeight() > 50) {
                		tmpIconxy = new ImageIcon(tmpIconx.getImage().
                                          getScaledInstance(-1, 50,
                                                      Image.SCALE_DEFAULT));
				logoImage = tmpIconxy;
                        } else { //no need to miniaturize
                                logoImage = tmpIcon;
                        }
                } else {
                        JOptionPane.showMessageDialog(new JFrame(), "Bad logo");
		}
	}

	private void save() {

		//put the image into the web app
		try {
			BufferedImage newLogo = ImageIO.read(new File(logo));
			File currLogo = new File(TestbedManager.getWebappPath() + "images/testbed_logo.jpg");
			currLogo.createNewFile();
			ImageIO.write(newLogo, "jpg", currLogo);
		} catch (Exception e) {
			JOptionPane.showMessageDialog(new JFrame(), "Error updating logo");
			return;
		}

		JOptionPane.showMessageDialog(new JFrame(), "Save Successful");
		dispose();
	}
}

