import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;
import java.io.*;
import javax.imageio.*;
import java.awt.image.*;

/**
 * Dialog to allow user to modify the Testbed map.
 *
 * @author      Hussein Gibbins
 */
public class MapDialog extends JDialog {
	
	JButton save;
	JButton cancel;
	String map = TestbedManager.getWebappPath()+"worldmap.jpg";
	JFileChooser mapfc;
	Properties thisProps = new Properties();
	TestbedManager tbm;
	boolean accept = false;

	ImageIcon mapImage = new ImageIcon();
	JLabel mapPreview = new JLabel(); 
	JButton mapButton;

        /**
         * Creates a MapDialog instance with the specified modality.  Will access
         * the testbed specified by the provided testbed manager.
         */
	public MapDialog(Frame owner, boolean modal, TestbedManager tbm) {
		super(owner, modal);
		this.tbm = tbm;
		init();
	}

	private void init() {

		// load thisprops first and then use getProperty for defaults.

		getContentPane().setLayout(new BorderLayout());
		JPanel options = new JPanel();
		getContentPane().add(options, BorderLayout.NORTH);
		options.setLayout(new GridLayout(2,1));

		// make new panel for map
		JPanel mapPanel = new JPanel();
		getContentPane().add(mapPanel, BorderLayout.CENTER);
		mapPanel.setLayout(new GridLayout(1, 2));

            mapPanel.add(new JLabel("Map"));

            //map preview
		updateMap(TestbedManager.getWebappPath() + "worldmap.jpg");
                mapButton = new JButton("Browse");
                mapButton.setIcon(mapImage);
                mapButton.setVerticalTextPosition(AbstractButton.BOTTOM);
                mapButton.setHorizontalTextPosition(AbstractButton.CENTER);
                mapPanel.add(mapButton);
                mapButton.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent event) {

                                //Set up the file chooser.
                                if (mapfc == null) {
                                        mapfc = new JFileChooser();

                                        //Add a custom file filter and disable the default
                                        //(Accept All) file filter.
                                        mapfc.addChoosableFileFilter(new ImageFilter());
                                        mapfc.setAcceptAllFileFilterUsed(false);

                                        //Add the preview pane.
                                        mapfc.setAccessory(new ImagePreview(mapfc));
                                }

                                //Show it.
                                int returnVal = mapfc.showDialog(new JFrame(), "Set");

                                //Process the results.
                                if (returnVal == JFileChooser.APPROVE_OPTION) {
                                        File file = mapfc.getSelectedFile();
					map = file.getPath();
                                        updateMap(file.getPath());
					mapButton.setIcon(mapImage);
                                } else {
                                        //System.out.println("Attachment cancelled by user.");
                                }

                                //Reset the file chooser for the next time it's shown.
                                mapfc.setSelectedFile(null);
                        }
                });

		//do buttons
		JPanel buttons = new JPanel();
		getContentPane().add(buttons, BorderLayout.SOUTH);
		buttons.setLayout(new BoxLayout(buttons, BoxLayout.X_AXIS));

                JButton save = new JButton("Save");
                buttons.add(save);
                save.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent event) {
					accept = true;
					save();
                        }
                });

                JButton cancel = new JButton("Cancel");
                buttons.add(cancel);
                cancel.addActionListener(new ActionListener() {
                        public void actionPerformed(ActionEvent event) {
				accept = false;
                                dispose();
                        }
                });

		this.pack();
	}


	private void updateMap(String mapPath) {

		ImageIcon tmpIcon = new ImageIcon(mapPath);
                ImageIcon tmpIconx = new ImageIcon(mapPath);
                ImageIcon tmpIconxy = new ImageIcon(mapPath);


                if (tmpIcon != null) {

			if (tmpIcon.getIconWidth() <= 90 && tmpIcon.getIconHeight() <= 50) {
                                mapImage = tmpIcon;
			} else {

	                        if (tmpIcon.getIconWidth() > 90) {
	                                tmpIconx = new ImageIcon(tmpIcon.getImage().
	                                                getScaledInstance(90, -1,
	                                                Image.SCALE_DEFAULT));
	                                mapImage = tmpIconx;
	                        } if (tmpIconx.getIconHeight() > 50) {
	                                tmpIconxy = new ImageIcon(tmpIconx.getImage().
	                                          getScaledInstance(-1, 50,
                                                      Image.SCALE_DEFAULT));
	                                mapImage = tmpIconxy;
                        	}
                        }
                } else {
			JOptionPane.showMessageDialog(new JFrame(), "Error updating map");
		}
	}

	private void save() {

		//put the image into the web app
		try {
			BufferedImage newMap = ImageIO.read(new File(map));
			File currMap = new File(TestbedManager.getWebappPath() + "worldmap.jpg");
			currMap.createNewFile();
			ImageIO.write(newMap, "jpg", currMap);
		} catch (Exception e) {
			JOptionPane.showMessageDialog(new JFrame(), "Error updating map");
			return;
		}

		JOptionPane.showMessageDialog(new JFrame(), "Save Successful");
		dispose();

	}

	/**
	 * Get the path of the image of the new map which was selected by the user.
	 *
	 * @return the path of the image which was selected.
	 */
	public String getMapPath() {
		return this.map;
	}

	/**
	 * Identifies whether the dialog was accepted or cancelled.
	 *
	 * @return whether the dialog was accepted.
	 */
	public boolean isAccepted() {
		return accept;
	}
}

