import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;
import java.io.*;

import javax.imageio.*;

/**
 * The class where the actual world map is defined, and the mouse clicks of
 * the user processed.
 *
 * @author Hussein Gibbins.
 */
public class MapPanel extends JLayeredPane implements MouseListener, MouseMotionListener, Observer {

	public static final int DETAILS = 0;
	public static final int ATTRIBUTES = 1;

	public static final int VIEWING = 0;
	public static final int EDITING = 1;
	private static final String DEFAULT_SEP = "|||";

	private JLabel backgroundImage;

	private int status = VIEWING;
	private Location tmpLoc = null;
	private LocationList locations;
	private TestbedManager tbm;

	private JFrame parent = null;

	private int tmpCounter = 0;

	private static JLabel theImage = new JLabel();

	Vector buttons = new Vector();
	int drag = 0;


        /**
         * Creates an instance of the MapPanel.
         */
	public MapPanel(TestbedManager tbm, JFrame parent) {

		this.tbm = tbm;
		this.parent = parent;
		locations = tbm.getLocationList();

		this.addMouseListener(this);

		File file = new File(TestbedManager.getWebappPath() + "worldmap.jpg");
		if (file.exists()) {
			ImageIcon icon = new ImageIcon(TestbedManager.getWebappPath() + "worldmap.jpg");
			backgroundImage = new JLabel(icon);

			backgroundImage.setBounds(0, 0, icon.getIconWidth(), icon.getIconHeight());
			this.add(backgroundImage, JLayeredPane.FRAME_CONTENT_LAYER);
			this.setMinimumSize(new Dimension(icon.getIconWidth(), icon.getIconHeight()));
			this.setPreferredSize(new Dimension(icon.getIconWidth(), icon.getIconHeight()));
			this.setMaximumSize(new Dimension(icon.getIconWidth(), icon.getIconHeight()));
		} else {

                        backgroundImage = new JLabel("Please Open a Testbed");
                        backgroundImage.setSize(new Dimension(200,50));
                        this.add(backgroundImage, JLayeredPane.FRAME_CONTENT_LAYER, 0);

                        this.setMinimumSize(new Dimension(200,200));
                        this.setPreferredSize(new Dimension(200,200));
                        this.setMaximumSize(new Dimension(200,200));
			this.repaint();

		}
	}

	private JButton makeButton(int x, int y) {
		JButton button = new JButton();
		button.setIcon(new ImageIcon("images/default.gif"));
		button.setSelectedIcon(new ImageIcon("images/selected.gif"));
		button.setPressedIcon(new ImageIcon("images/pressed.gif"));
		button.setBorderPainted(false);
		button.setMargin(new Insets(0,0,0,0));
		button.setPreferredSize(new Dimension(10,10));
		button.addMouseListener(this);
		button.addMouseMotionListener(this);
		button.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {

				doSelected((JButton)event.getSource());
				if(status == VIEWING) {
					tbm.queryTop();
				}

			}
            });
		button.setBounds(x,y,10,10);

		return button;
	}

	private void doSelected(JButton selected) {

		JButton b;
		int index;

		for (index = 0; index < buttons.size(); index++) {
			b = (JButton) buttons.elementAt(index);
			if (b.isSelected()) {
				b.setSelected(false);
				break;
			}
		}

		selected.setSelected(true);
		locations.moveToTop(locations.elementAt(buttons.indexOf(selected)));
		refresh();
	}


	/**
	 * Used to allow other components to tell the map panel to refresh its contents.
	 */
	public void refresh() {

		for (int h = 0; h < buttons.size(); h++) {
			this.remove((JButton)buttons.elementAt(h));
		}

		buttons.clear();

		buttons = new Vector();
		JButton b;

		for (int i = 0; i < locations.size(); i++) {

                        Location newLoc = (Location) locations.elementAt(i);
				b = makeButton(newLoc.getX(), newLoc.getY());
                        this.add(b, JLayeredPane.DEFAULT_LAYER, 0);
				buttons.add(i, b);
                        this.repaint();
		}
		
		try {
			((JButton)buttons.firstElement()).setSelected(true);
		} catch (NoSuchElementException e) {
			// Do nothing. Nothing to select
		}
	}

	/**
	 * Changes the background image displayed by this panel.
	 *
	 * @param newMapPath string indicating the path to the new image.
	 */
	public void setBackgroundMap(String newMapPath) {

		this.remove(backgroundImage);
		
		ImageIcon icon = new ImageIcon(newMapPath);
                backgroundImage = new JLabel(icon);
                backgroundImage.setBounds(0, 0, icon.getIconWidth(), icon.getIconHeight());
                this.add(backgroundImage, JLayeredPane.FRAME_CONTENT_LAYER);
                this.setMinimumSize(new Dimension(icon.getIconWidth(), icon.getIconHeight()));
                this.setPreferredSize(new Dimension(icon.getIconWidth(), icon.getIconHeight()));
                this.setMaximumSize(new Dimension(icon.getIconWidth(), icon.getIconHeight()));

                parent.pack();
                this.repaint();
	}

	public void doUpdate() {

		setBackgroundMap(TestbedManager.getWebappPath() + "worldmap.jpg");
		this.refresh();
	}

	/**
	 * Updates the contents of this panel in response to changes in objects this is listening to.
	 *
	 * @see Observable
	 */
	public void update(Observable o, Object args) {
		if (args != null) {
			if (((String)args).equals("complete")) {
				doUpdate();
			} else if (((String)args).equals("remove")) {
				this.remove((JButton)buttons.firstElement());
				buttons.remove(buttons.firstElement());
				if (buttons.size() > 0) {
					doSelected((JButton)buttons.firstElement());
				}
				this.repaint();
			}
		}
	}

	/**
	 * Indicates to this panel which mode the application is currently running in.
	 *
	 * @param status the status/mode of the application.
	 */
	public void setStatus(int status) {
		if (status <= 1) {
			this.status = status;
		}
	}


	// not used
	public void mouseClicked(MouseEvent e) {
	}

	// not used
	public void mouseEntered(MouseEvent e) {
	}

	// not used
	public void mouseExited(MouseEvent e) {
	}

	public void mousePressed(MouseEvent e) {

		if (e.getSource() instanceof JButton) {
			drag = buttons.indexOf(e.getSource());
			Location loc = (Location)locations.elementAt(drag);
			Location newLoc = new Location(loc.getName(), loc.getAddress(), loc.getPort(), loc.getDetails(), loc.getX(), loc.getY());
			newLoc.setCountryData(loc.getCountryCode() + ":" + loc.getCountryName());
			locations.set(drag, newLoc);
			
			
		} else {
			if (status == EDITING) {
				String address = " ";
				int port = 2135;
				Location theNewLoc = new Location("[" + tmpCounter + "] Enter Grid Name", address, port, "Description", e.getX()-5, e.getY()-5);
				locations.add(theNewLoc);
				tmpCounter++;
				refresh();
			}
		}
	}

	public void mouseReleased(MouseEvent e) {

	}

	public void mouseDragged(MouseEvent e) {

		if (status == EDITING) {

			if (e.getSource() instanceof JButton) {

				Location loc = (Location)locations.elementAt(drag);
				loc.setX(((JButton)e.getSource()).getX()+e.getX()-5);
				loc.setY(((JButton)e.getSource()).getY()+e.getY()-5);

				((JButton)e.getSource()).setLocation(((JButton)e.getSource()).getX()+e.getX()-5,((JButton)e.getSource()).getY()+e.getY()-5);
			}
		}
	}

	public void mouseMoved(MouseEvent e) {

	}
}
