import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.lang.*;
import java.util.*;
import java.io.*;
import javax.imageio.*;
import java.awt.image.*;

/**
 * Dialog to allow user to modify the name of the Testbed.
 *
 * @author	Hussein Gibbins
 */
public class NameDialog extends JDialog {
	
	JButton save;
	JButton cancel;
	JTextField name = new JTextField(TestbedManager.getTestbedName());

	Properties thisProps = new Properties();
	TestbedManager tbm;

	/**
         * Creates a NameDialog instance with the specified modality.  Will access
	 * the testbed specified by the provided testbed manager.
	 */
	public NameDialog(Frame owner, boolean modal, TestbedManager tbm) {
		super(owner, modal);
		this.tbm = tbm;
		init();
	}

	private void init() {

		// load thisprops first and then use getProperty for defaults.

		getContentPane().setLayout(new BorderLayout());
		JPanel options = new JPanel();
		getContentPane().add(options, BorderLayout.NORTH);
		options.setLayout(new GridLayout(2,1));

		// do web app name
		options.add(new JLabel("Name of Testbed"));
		name.setColumns(10);
		options.add(name);

		// do buttons
		JPanel buttons = new JPanel();
		getContentPane().add(buttons, BorderLayout.SOUTH);
		buttons.setLayout(new BoxLayout(buttons, BoxLayout.X_AXIS));

		JButton save = new JButton("Save");
		buttons.add(save);
		save.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				save();
			}
		});

		JButton cancel = new JButton("Cancel");
		buttons.add(cancel);
		cancel.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent event) {
				dispose();
			}
		});

		//load thisProperties
                try {
                        thisProps.load(new FileInputStream(TestbedManager.getWebappPath() 
						+ "WEB-INF/classes/gridslab/testbed.properties"));
                } catch (FileNotFoundException fnfe) {
                        JOptionPane.showMessageDialog(NameDialog.this,
                                "Failed to open location (" + TestbedManager.getWebappPath() 
						+ "WEB-INF/classes/gridslab/testbed.properties).");
                        return;
                } catch (IOException ioe) {
                        JOptionPane.showMessageDialog(NameDialog.this,
                                "Error with properties file (testbed.properties)");
                        return;
                }

		name.setText(thisProps.getProperty("testbed_name"));

		this.pack();
	}


	private void save() {

		String nameText = name.getText();

		//set any properties for Admin Tool 
		thisProps.setProperty("testbed_name", nameText);
		
		//write any property changes to properties file (web app and stampede)
		try {
			thisProps.store(new FileOutputStream(TestbedManager.getWebappPath() 
				+ "WEB-INF/classes/gridslab/testbed.properties"), "Testbed Properties");
		} catch (Exception e) {
			JOptionPane.showMessageDialog(new JFrame(), "error writing properties");
			return;
		}

		JOptionPane.showMessageDialog(new JFrame(), "Save Successful");
		dispose();
	}
}

